"""This script reads the binary file containing the hyperspectral
images and writes the images to an ENVI file.
Also as the data is packed in Mono12p format,
it unpacks the data to 16-bit unsigned integer format.
"""

from __future__ import annotations

import csv
import struct
from pathlib import Path
from typing import TYPE_CHECKING, Any

import numpy as np
from spectral.io import envi

from phenomate_core.preprocessing.base import BasePreprocessor
from phenomate_core.preprocessing.hyperspec import hyperspec_pb2 as hs_pb2

if TYPE_CHECKING:
    from numpy.typing import NDArray

DEFAULT_WIDTH = 1024
DEFAULT_HEIGHT = 224


class HyperspecPreprocessor(BasePreprocessor[hs_pb2.HyperSpecImage]):
    def extract(self, **kwargs: Any) -> None:
        with self.path.open("rb") as file:
            while True:
                # Read the length of the next serialized message
                serialized_timestamp = file.read(8)
                if not serialized_timestamp:
                    break
                system_timestamp = struct.unpack("d", serialized_timestamp)[0]

                length_bytes = file.read(4)
                if not length_bytes:
                    break
                length = int.from_bytes(length_bytes, byteorder="little")

                # Read the serialized message
                serialized_image = file.read(length)

                # Parse the protobuf message
                image_protobuf_obj = hs_pb2.HyperSpecImage()
                image_protobuf_obj.ParseFromString(serialized_image)

                # amiga_timestamp = image_protobuf_obj.timestamp_info

                # # Convert the image data back to numpy.ndarray
                self.images.append(image_protobuf_obj)
                self.system_timestamps.append(system_timestamp)

    @staticmethod
    def unpack_mono12packed_to_16bit(
        packed_data: NDArray[np.uint8], width: int, height: int
    ) -> NDArray[np.uint16]:
        """Unpack Mono12p packed data to 16-bit unsigned integer format."""

        # Reshape packed_data to separate each set of 3 bytes
        packed_data = packed_data.reshape(-1, 3)

        # Unpack the first 12 bits into the first 16-bit value
        first_pixels = (packed_data[:, 0].astype(np.uint16) << 4) | (packed_data[:, 1] >> 4)

        # Unpack the remaining 12 bits into the second 16-bit value
        second_pixels = (packed_data[:, 2].astype(np.uint16) << 4) | (packed_data[:, 1] & 0x0F)

        # Interleave the unpacked data into a single array
        unpacked_data = np.zeros(width * height, dtype=np.uint16)
        unpacked_data[0::2] = first_pixels
        unpacked_data[1::2] = second_pixels

        # Reshape to height, width
        return unpacked_data[: width * height].reshape(height, width)

    def write_to_envi_file(self, path: Path, width: int, height: int) -> None:
        """This function writes the hyperspectral images to an ENVI file."""
        envi_filename = path / self.get_output_name(None, "hdr")

        num_images = len(self.images) - 1  # Number of images
        num_samples = self.images[0].width
        num_bands = self.images[0].height

        md = {
            "lines": num_images,  # Rows
            "samples": num_samples,  # Columns
            "bands": num_bands,  # image's spectral dimensionality
            "data type": 12,  # ENVI data type for 16-bit integer
            "interleave": "bil",
            "byte order": 1,
        }
        envi_image = envi.create_image(envi_filename, md, interleave="bil", ext="raw")  # type: ignore[no-untyped-call]

        envi_memmap = envi_image.open_memmap(interleave="bil", writable=True)

        for index in range(num_images):
            packed_data = self.bytes_to_numpy(self.images[index].image_data)
            unpacked_data = self.unpack_mono12packed_to_16bit(packed_data, width, height)
            envi_memmap[index, :, :] = unpacked_data

    def write_to_csv_file(self, path: Path, **kwargs: Any) -> None:
        """This function writes the information of the hyperspectral images
        to a CSV file.
        """
        headers = [
            "system_timestamp",
            "width",
            "height",
            "frame_rate",
            "blockid",
            "bandwidth",
            "image_timestamp",
        ]
        file_path = path / self.get_output_name(None, "csv")
        with file_path.open("w", encoding="utf-8") as csv_file:
            writer = csv.writer(csv_file)
            writer.writerow(headers)
            for system_timestamp, image in zip(self.system_timestamps, self.images, strict=False):
                writer.writerow(
                    [
                        system_timestamp,
                        image.width,
                        image.height,
                        image.frame_rate,
                        image.blockid,
                        image.bandwidth,
                        image.timestamp,
                    ]
                )

    def save(
        self,
        path: Path | str,
        width: int = DEFAULT_WIDTH,
        height: int = DEFAULT_HEIGHT,
        **kwargs: Any,
    ) -> None:
        file_path = Path(path)
        file_path.mkdir(parents=True, exist_ok=True)
        self.write_to_csv_file(file_path)
        self.write_to_envi_file(file_path, width, height)



from phenomate_core.preprocessing.hyperspec.process import HyperspecPreprocessor
from phenomate_core.preprocessing.jai.process import JaiPreprocessor
from phenomate_core.preprocessing.oak_d.process import (
    OakCalibrationPreprocessor,
    OakFramePreprocessor,
    OakImuPacketsPreprocessor,
)

__all__ = (
    "HyperspecPreprocessor",
    "JaiPreprocessor",
    "OakCalibrationPreprocessor",
    "OakFramePreprocessor",
    "OakImuPacketsPreprocessor",
)



from __future__ import annotations

import struct
from pathlib import Path
from typing import Any

from PIL import Image

from phenomate_core.preprocessing.base import BasePreprocessor
from phenomate_core.preprocessing.jai import jai_pb2


class JaiPreprocessor(BasePreprocessor[jai_pb2.JAIImage]):
    def extract(self, **kwargs: Any) -> None:
        with self.path.open("rb") as file:
            while True:
                # Read the length of the next serialized message
                serialized_timestamp = file.read(8)
                if not serialized_timestamp:
                    break
                system_timestamp = struct.unpack("d", serialized_timestamp)[0]

                length_bytes = file.read(4)
                if not length_bytes:
                    break
                length = int.from_bytes(length_bytes, byteorder="little")

                # Read the serialized message
                serialized_image = file.read(length)

                # Parse the protobuf message
                image_protobuf_obj = jai_pb2.JAIImage()
                image_protobuf_obj.ParseFromString(serialized_image)

                # Update to extracted image list
                self.images.append(image_protobuf_obj)
                self.system_timestamps.append(system_timestamp)

    def save(
        self,
        path: Path | str,
        width: int | None = None,
        height: int | None = None,
        **kwargs: Any,
    ) -> None:
        fpath = Path(path)
        fpath.mkdir(parents=True, exist_ok=True)
        for index, image in enumerate(self.images):
            # Determine width and height
            iwidth = width if width is not None else image.width
            iheight = height if height is not None else image.height
            reshaped_image = self.bytes_to_numpy(image.image_data).reshape((iheight, iwidth))
            # Convert the reshaped image data to a PIL Image object
            out_image = Image.fromarray(reshaped_image)
            image_path = fpath / self.get_output_name(index, "jpeg")
            out_image.save(image_path)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: oak.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'oak.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\toak.proto\"\x1f\n\x07Vec2I64\x12\t\n\x01x\x18\x01 \x01(\x03\x12\t\n\x01y\x18\x02 \x01(\x03\"\x1f\n\x07Vec2F32\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\"\x1f\n\x07Vec2F64\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\"*\n\x07Vec3I64\x12\t\n\x01x\x18\x01 \x01(\x03\x12\t\n\x01y\x18\x02 \x01(\x03\x12\t\n\x01z\x18\x03 \x01(\x03\"*\n\x07Vec3F32\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"*\n\x07Vec3F64\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\t\n\x01z\x18\x03 \x01(\x01\";\n\x07Mat2F32\x12\x17\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x08.Vec2F32\x12\x17\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x08.Vec2F32\";\n\x07Mat2F64\x12\x17\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x08.Vec2F64\x12\x17\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x08.Vec2F64\"T\n\x07Mat3F32\x12\x17\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x08.Vec3F32\x12\x17\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x08.Vec3F32\x12\x17\n\x05\x63ol_2\x18\x03 \x01(\x0b\x32\x08.Vec3F32\"T\n\x07Mat3F64\x12\x17\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x08.Vec3F64\x12\x17\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x08.Vec3F64\x12\x17\n\x05\x63ol_2\x18\x03 \x01(\x0b\x32\x08.Vec3F64\"\x17\n\x07VecXF32\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x02\"\x17\n\x07VecXF64\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x01\"3\n\x07MatXF32\x12\x0c\n\x04rows\x18\x01 \x01(\x05\x12\x0c\n\x04\x63ols\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x02\"3\n\x07MatXF64\x12\x0c\n\x04rows\x18\x01 \x01(\x05\x12\x0c\n\x04\x63ols\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x02\"c\n\x0e\x43\x61meraSettings\x12\x15\n\rauto_exposure\x18\x01 \x01(\x08\x12\x15\n\rexposure_time\x18\x02 \x01(\r\x12\x11\n\tiso_value\x18\x03 \x01(\r\x12\x10\n\x08lens_pos\x18\x04 \x01(\r\"\xbe\x01\n\x0e\x45ncoderOptions\x12 \n\x07profile\x18\x01 \x01(\x0e\x32\x0f.EncoderProfile\x12+\n\x11rate_control_mode\x18\x02 \x01(\x0e\x32\x10.RateControlMode\x12\"\n\x1a\x63\x62r_preferred_bitrate_kbps\x18\x03 \x01(\r\x12\x1c\n\x14vbr_or_mjpeg_quality\x18\x04 \x01(\r\x12\x1b\n\x13\x66rames_per_keyframe\x18\x05 \x01(\r\"+\n\nResolution\x12\r\n\x05width\x18\x01 \x01(\r\x12\x0e\n\x06height\x18\x02 \x01(\r\"\xa3\x02\n\x0cOakImageMeta\x12\x10\n\x08\x63\x61tegory\x18\x01 \x01(\x03\x12\x14\n\x0cinstance_num\x18\x02 \x01(\x03\x12\x14\n\x0csequence_num\x18\x03 \x01(\x03\x12\x11\n\ttimestamp\x18\x04 \x01(\x01\x12\x18\n\x10timestamp_device\x18\x05 \x01(\x01\x12\x16\n\x0etimestamp_recv\x18\x07 \x01(\x01\x12!\n\x08settings\x18\x06 \x01(\x0b\x32\x0f.CameraSettings\x12(\n\x0f\x65ncoder_options\x18\x08 \x01(\x0b\x32\x0f.EncoderOptions\x12\x1f\n\nresolution\x18\t \x01(\x0b\x32\x0b.Resolution\x12\"\n\x0cpixel_format\x18\n \x01(\x0e\x32\x0c.PixelFormat\";\n\x08OakFrame\x12\x1b\n\x04meta\x18\x01 \x01(\x0b\x32\r.OakImageMeta\x12\x12\n\nimage_data\x18\x02 \x01(\x0c\"\x8e\x01\n\x07OakGyro\x12\x16\n\x04gyro\x18\x01 \x01(\x0b\x32\x08.Vec3F32\x12\x14\n\x0csequence_num\x18\x02 \x01(\x05\x12\x10\n\x08\x61\x63\x63uracy\x18\x03 \x01(\t\x12\x11\n\ttimestamp\x18\x04 \x01(\x01\x12\x18\n\x10timestamp_device\x18\x05 \x01(\x01\x12\x16\n\x0etimestamp_recv\x18\x06 \x01(\x01\"\x96\x01\n\x0bOakAccelero\x12\x1a\n\x08\x61\x63\x63\x65lero\x18\x01 \x01(\x0b\x32\x08.Vec3F32\x12\x14\n\x0csequence_num\x18\x02 \x01(\x05\x12\x10\n\x08\x61\x63\x63uracy\x18\x03 \x01(\t\x12\x11\n\ttimestamp\x18\x04 \x01(\x01\x12\x18\n\x10timestamp_device\x18\x05 \x01(\x01\x12\x16\n\x0etimestamp_recv\x18\x06 \x01(\x01\"T\n\x0cOakImuPacket\x12\x1d\n\x0bgyro_packet\x18\x01 \x01(\x0b\x32\x08.OakGyro\x12%\n\x0f\x61\x63\x63\x65lero_packet\x18\x02 \x01(\x0b\x32\x0c.OakAccelero\"/\n\rOakImuPackets\x12\x1e\n\x07packets\x18\x01 \x03(\x0b\x32\r.OakImuPacket\"t\n\x17OakTrackedFeaturePacket\x12\x14\n\x02xy\x18\x01 \x01(\x0b\x32\x08.Vec2F32\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x0b\n\x03\x61ge\x18\x03 \x01(\x05\x12\x13\n\x0bharrisScore\x18\x04 \x01(\x02\x12\x15\n\rtrackingError\x18\x05 \x01(\x02\"\xa0\x01\n\x18OakTrackedFeaturePackets\x12)\n\x07packets\x18\x01 \x03(\x0b\x32\x18.OakTrackedFeaturePacket\x12\x14\n\x0csequence_num\x18\x02 \x01(\x03\x12\x11\n\ttimestamp\x18\x03 \x01(\x01\x12\x18\n\x10timestamp_device\x18\x04 \x01(\x01\x12\x16\n\x0etimestamp_recv\x18\x05 \x01(\x01\"7\n\rOakDeviceInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04mxid\x18\x02 \x01(\t\x12\n\n\x02ip\x18\x03 \x01(\t\"\xef\x01\n\x0cOakSyncFrame\x12\x17\n\x04left\x18\x01 \x01(\x0b\x32\t.OakFrame\x12\x18\n\x05right\x18\x02 \x01(\x0b\x32\t.OakFrame\x12\x16\n\x03rgb\x18\x03 \x01(\x0b\x32\t.OakFrame\x12\x1c\n\tdisparity\x18\x04 \x01(\x0b\x32\t.OakFrame\x12\x16\n\x02nn\x18\x05 \x01(\x0b\x32\n.OakNNData\x12#\n\x0bimu_packets\x18\x06 \x01(\x0b\x32\x0e.OakImuPackets\x12\x14\n\x0csequence_num\x18\x07 \x01(\x03\x12#\n\x0b\x64\x65vice_info\x18\x08 \x01(\x0b\x32\x0e.OakDeviceInfo\"k\n\tOakNNData\x12\x1b\n\x04meta\x18\x01 \x01(\x0b\x32\r.OakImageMeta\x12\x14\n\x0cnum_channels\x18\x02 \x01(\r\x12\x0e\n\x06height\x18\x03 \x01(\r\x12\r\n\x05width\x18\x04 \x01(\r\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\"\"\n\x04Pair\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\" \n\x08Metadata\x12\x14\n\x05pairs\x18\x01 \x03(\x0b\x32\x05.Pair\"J\n\rOakDataSample\x12\x1c\n\x05\x66rame\x18\x01 \x01(\x0b\x32\r.OakSyncFrame\x12\x1b\n\x08metadata\x18\x02 \x01(\x0b\x32\t.Metadata\")\n\x0eRotationMatrix\x12\x17\n\x0frotation_matrix\x18\x01 \x03(\x01\"\x82\x01\n\nExtrinsics\x12\x17\n\x0frotation_matrix\x18\x01 \x03(\x01\x12\"\n\x10spec_translation\x18\x02 \x01(\x0b\x32\x08.Vec3F32\x12\x18\n\x10to_camera_socket\x18\x03 \x01(\x05\x12\x1d\n\x0btranslation\x18\x04 \x01(\x0b\x32\x08.Vec3F32\"\xda\x01\n\nCameraData\x12\x15\n\rcamera_number\x18\x01 \x01(\r\x12\x13\n\x0b\x63\x61mera_type\x18\x02 \x01(\x05\x12\x18\n\x10\x64istortion_coeff\x18\x03 \x03(\x01\x12\x1f\n\nextrinsics\x18\x04 \x01(\x0b\x32\x0b.Extrinsics\x12\x0e\n\x06height\x18\x05 \x01(\r\x12\x18\n\x10intrinsic_matrix\x18\x06 \x03(\x01\x12\x15\n\rlens_position\x18\x07 \x01(\r\x12\x15\n\rspec_hfov_deg\x18\x08 \x01(\x01\x12\r\n\x05width\x18\t \x01(\r\"\x95\x01\n\x17StereoRectificationData\x12\x1a\n\x12left_camera_socket\x18\x01 \x01(\r\x12\x1f\n\x17rectified_rotation_left\x18\x02 \x03(\x01\x12 \n\x18rectified_rotation_right\x18\x03 \x03(\x01\x12\x1b\n\x13right_camera_socket\x18\x04 \x01(\r\"\xfe\x02\n\x0eOakCalibration\x12\x12\n\nbatch_name\x18\x01 \x01(\t\x12\x12\n\nbatch_time\x18\x02 \x01(\x05\x12\x12\n\nboard_conf\x18\x03 \x01(\t\x12\x14\n\x0c\x62oard_custom\x18\x04 \x01(\t\x12\x12\n\nboard_name\x18\x05 \x01(\t\x12\x15\n\rboard_options\x18\x06 \x01(\x05\x12\x11\n\tboard_rev\x18\x07 \x01(\t\x12 \n\x0b\x63\x61mera_data\x18\x08 \x03(\x0b\x32\x0b.CameraData\x12\x15\n\rhardware_conf\x18\t \x01(\t\x12#\n\x0eimu_extrinsics\x18\n \x01(\x0b\x32\x0b.Extrinsics\x12\x1a\n\x12miscellaneous_data\x18\x0b \x03(\t\x12\x14\n\x0cproduct_name\x18\x0c \x01(\t\x12;\n\x19stereo_rectification_data\x18\r \x01(\x0b\x32\x18.StereoRectificationData\x12\x0f\n\x07version\x18\x0e \x01(\r*[\n\x0e\x45ncoderProfile\x12\x11\n\rH264_BASELINE\x10\x00\x12\r\n\tH264_HIGH\x10\x01\x12\r\n\tH264_MAIN\x10\x02\x12\r\n\tH265_MAIN\x10\x03\x12\t\n\x05MJPEG\x10\x04*#\n\x0fRateControlMode\x12\x07\n\x03\x43\x42R\x10\x00\x12\x07\n\x03VBR\x10\x01*=\n\x0bPixelFormat\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06YUV420\x10\x01\x12\x08\n\x04RAW8\x10\x02\x12\x0b\n\x07\x45ncoded\x10\x03\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'oak_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ENCODERPROFILE']._serialized_start=3590
  _globals['_ENCODERPROFILE']._serialized_end=3681
  _globals['_RATECONTROLMODE']._serialized_start=3683
  _globals['_RATECONTROLMODE']._serialized_end=3718
  _globals['_PIXELFORMAT']._serialized_start=3720
  _globals['_PIXELFORMAT']._serialized_end=3781
  _globals['_VEC2I64']._serialized_start=13
  _globals['_VEC2I64']._serialized_end=44
  _globals['_VEC2F32']._serialized_start=46
  _globals['_VEC2F32']._serialized_end=77
  _globals['_VEC2F64']._serialized_start=79
  _globals['_VEC2F64']._serialized_end=110
  _globals['_VEC3I64']._serialized_start=112
  _globals['_VEC3I64']._serialized_end=154
  _globals['_VEC3F32']._serialized_start=156
  _globals['_VEC3F32']._serialized_end=198
  _globals['_VEC3F64']._serialized_start=200
  _globals['_VEC3F64']._serialized_end=242
  _globals['_MAT2F32']._serialized_start=244
  _globals['_MAT2F32']._serialized_end=303
  _globals['_MAT2F64']._serialized_start=305
  _globals['_MAT2F64']._serialized_end=364
  _globals['_MAT3F32']._serialized_start=366
  _globals['_MAT3F32']._serialized_end=450
  _globals['_MAT3F64']._serialized_start=452
  _globals['_MAT3F64']._serialized_end=536
  _globals['_VECXF32']._serialized_start=538
  _globals['_VECXF32']._serialized_end=561
  _globals['_VECXF64']._serialized_start=563
  _globals['_VECXF64']._serialized_end=586
  _globals['_MATXF32']._serialized_start=588
  _globals['_MATXF32']._serialized_end=639
  _globals['_MATXF64']._serialized_start=641
  _globals['_MATXF64']._serialized_end=692
  _globals['_CAMERASETTINGS']._serialized_start=694
  _globals['_CAMERASETTINGS']._serialized_end=793
  _globals['_ENCODEROPTIONS']._serialized_start=796
  _globals['_ENCODEROPTIONS']._serialized_end=986
  _globals['_RESOLUTION']._serialized_start=988
  _globals['_RESOLUTION']._serialized_end=1031
  _globals['_OAKIMAGEMETA']._serialized_start=1034
  _globals['_OAKIMAGEMETA']._serialized_end=1325
  _globals['_OAKFRAME']._serialized_start=1327
  _globals['_OAKFRAME']._serialized_end=1386
  _globals['_OAKGYRO']._serialized_start=1389
  _globals['_OAKGYRO']._serialized_end=1531
  _globals['_OAKACCELERO']._serialized_start=1534
  _globals['_OAKACCELERO']._serialized_end=1684
  _globals['_OAKIMUPACKET']._serialized_start=1686
  _globals['_OAKIMUPACKET']._serialized_end=1770
  _globals['_OAKIMUPACKETS']._serialized_start=1772
  _globals['_OAKIMUPACKETS']._serialized_end=1819
  _globals['_OAKTRACKEDFEATUREPACKET']._serialized_start=1821
  _globals['_OAKTRACKEDFEATUREPACKET']._serialized_end=1937
  _globals['_OAKTRACKEDFEATUREPACKETS']._serialized_start=1940
  _globals['_OAKTRACKEDFEATUREPACKETS']._serialized_end=2100
  _globals['_OAKDEVICEINFO']._serialized_start=2102
  _globals['_OAKDEVICEINFO']._serialized_end=2157
  _globals['_OAKSYNCFRAME']._serialized_start=2160
  _globals['_OAKSYNCFRAME']._serialized_end=2399
  _globals['_OAKNNDATA']._serialized_start=2401
  _globals['_OAKNNDATA']._serialized_end=2508
  _globals['_PAIR']._serialized_start=2510
  _globals['_PAIR']._serialized_end=2544
  _globals['_METADATA']._serialized_start=2546
  _globals['_METADATA']._serialized_end=2578
  _globals['_OAKDATASAMPLE']._serialized_start=2580
  _globals['_OAKDATASAMPLE']._serialized_end=2654
  _globals['_ROTATIONMATRIX']._serialized_start=2656
  _globals['_ROTATIONMATRIX']._serialized_end=2697
  _globals['_EXTRINSICS']._serialized_start=2700
  _globals['_EXTRINSICS']._serialized_end=2830
  _globals['_CAMERADATA']._serialized_start=2833
  _globals['_CAMERADATA']._serialized_end=3051
  _globals['_STEREORECTIFICATIONDATA']._serialized_start=3054
  _globals['_STEREORECTIFICATIONDATA']._serialized_end=3203
  _globals['_OAKCALIBRATION']._serialized_start=3206
  _globals['_OAKCALIBRATION']._serialized_end=3588
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: jai.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'jai.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tjai.proto\"\x88\x01\n\x08JAIImage\x12\x12\n\nimage_data\x18\x01 \x01(\x0c\x12\r\n\x05width\x18\x02 \x01(\r\x12\x0e\n\x06height\x18\x03 \x01(\r\x12\x12\n\nframe_rate\x18\x04 \x01(\x01\x12\x0f\n\x07\x62lockid\x18\x05 \x01(\x04\x12\x11\n\tbandwidth\x18\x06 \x01(\x04\x12\x11\n\ttimestamp\x18\x07 \x01(\x04\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'jai_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_JAIIMAGE']._serialized_start=14
  _globals['_JAIIMAGE']._serialized_end=150
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright (c) farm-ng, inc.

Licensed under the Amiga Development Kit License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://github.com/farm-ng/amiga-dev-kit/blob/main/LICENSE

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _EncoderProfile:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EncoderProfileEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EncoderProfile.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    H264_BASELINE: _EncoderProfile.ValueType  # 0
    H264_HIGH: _EncoderProfile.ValueType  # 1
    H264_MAIN: _EncoderProfile.ValueType  # 2
    H265_MAIN: _EncoderProfile.ValueType  # 3
    MJPEG: _EncoderProfile.ValueType  # 4

class EncoderProfile(_EncoderProfile, metaclass=_EncoderProfileEnumTypeWrapper):
    """The encoding format for the video stream
    https://docs.luxonis.com/projects/api/en/latest/references/python/#depthai.VideoEncoderProperties.Profile
    """

H264_BASELINE: EncoderProfile.ValueType  # 0
H264_HIGH: EncoderProfile.ValueType  # 1
H264_MAIN: EncoderProfile.ValueType  # 2
H265_MAIN: EncoderProfile.ValueType  # 3
MJPEG: EncoderProfile.ValueType  # 4
global___EncoderProfile = EncoderProfile

class _RateControlMode:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _RateControlModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_RateControlMode.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    CBR: _RateControlMode.ValueType  # 0
    """Constant Bit Rate"""
    VBR: _RateControlMode.ValueType  # 1
    """Variable Bit Rate"""

class RateControlMode(_RateControlMode, metaclass=_RateControlModeEnumTypeWrapper):
    """The rate control mode for the video stream
    https://docs.luxonis.com/projects/api/en/latest/references/python/#depthai.VideoEncoderProperties.RateControlMode
    """

CBR: RateControlMode.ValueType  # 0
"""Constant Bit Rate"""
VBR: RateControlMode.ValueType  # 1
"""Variable Bit Rate"""
global___RateControlMode = RateControlMode

class _PixelFormat:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _PixelFormatEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_PixelFormat.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _PixelFormat.ValueType  # 0
    YUV420: _PixelFormat.ValueType  # 1
    RAW8: _PixelFormat.ValueType  # 2
    Encoded: _PixelFormat.ValueType  # 3

class PixelFormat(_PixelFormat, metaclass=_PixelFormatEnumTypeWrapper): ...

UNKNOWN: PixelFormat.ValueType  # 0
YUV420: PixelFormat.ValueType  # 1
RAW8: PixelFormat.ValueType  # 2
Encoded: PixelFormat.ValueType  # 3
global___PixelFormat = PixelFormat

@typing.final
class Vec2I64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    x: builtins.int
    y: builtins.int
    def __init__(
        self,
        *,
        x: builtins.int = ...,
        y: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> None: ...

global___Vec2I64 = Vec2I64

@typing.final
class Vec2F32(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> None: ...

global___Vec2F32 = Vec2F32

@typing.final
class Vec2F64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> None: ...

global___Vec2F64 = Vec2F64

@typing.final
class Vec3I64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.int
    y: builtins.int
    z: builtins.int
    def __init__(
        self,
        *,
        x: builtins.int = ...,
        y: builtins.int = ...,
        z: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___Vec3I64 = Vec3I64

@typing.final
class Vec3F32(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
        z: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___Vec3F32 = Vec3F32

@typing.final
class Vec3F64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
        z: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___Vec3F64 = Vec3F64

@typing.final
class Mat2F32(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COL_0_FIELD_NUMBER: builtins.int
    COL_1_FIELD_NUMBER: builtins.int
    @property
    def col_0(self) -> global___Vec2F32: ...
    @property
    def col_1(self) -> global___Vec2F32: ...
    def __init__(
        self,
        *,
        col_0: global___Vec2F32 | None = ...,
        col_1: global___Vec2F32 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1"]) -> None: ...

global___Mat2F32 = Mat2F32

@typing.final
class Mat2F64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COL_0_FIELD_NUMBER: builtins.int
    COL_1_FIELD_NUMBER: builtins.int
    @property
    def col_0(self) -> global___Vec2F64: ...
    @property
    def col_1(self) -> global___Vec2F64: ...
    def __init__(
        self,
        *,
        col_0: global___Vec2F64 | None = ...,
        col_1: global___Vec2F64 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1"]) -> None: ...

global___Mat2F64 = Mat2F64

@typing.final
class Mat3F32(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COL_0_FIELD_NUMBER: builtins.int
    COL_1_FIELD_NUMBER: builtins.int
    COL_2_FIELD_NUMBER: builtins.int
    @property
    def col_0(self) -> global___Vec3F32: ...
    @property
    def col_1(self) -> global___Vec3F32: ...
    @property
    def col_2(self) -> global___Vec3F32: ...
    def __init__(
        self,
        *,
        col_0: global___Vec3F32 | None = ...,
        col_1: global___Vec3F32 | None = ...,
        col_2: global___Vec3F32 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1", "col_2", b"col_2"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1", "col_2", b"col_2"]) -> None: ...

global___Mat3F32 = Mat3F32

@typing.final
class Mat3F64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COL_0_FIELD_NUMBER: builtins.int
    COL_1_FIELD_NUMBER: builtins.int
    COL_2_FIELD_NUMBER: builtins.int
    @property
    def col_0(self) -> global___Vec3F64: ...
    @property
    def col_1(self) -> global___Vec3F64: ...
    @property
    def col_2(self) -> global___Vec3F64: ...
    def __init__(
        self,
        *,
        col_0: global___Vec3F64 | None = ...,
        col_1: global___Vec3F64 | None = ...,
        col_2: global___Vec3F64 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1", "col_2", b"col_2"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["col_0", b"col_0", "col_1", b"col_1", "col_2", b"col_2"]) -> None: ...

global___Mat3F64 = Mat3F64

@typing.final
class VecXF32(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        data: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___VecXF32 = VecXF32

@typing.final
class VecXF64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        data: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___VecXF64 = VecXF64

@typing.final
class MatXF32(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    COLS_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    rows: builtins.int
    cols: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        rows: builtins.int = ...,
        cols: builtins.int = ...,
        data: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["cols", b"cols", "data", b"data", "rows", b"rows"]) -> None: ...

global___MatXF32 = MatXF32

@typing.final
class MatXF64(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROWS_FIELD_NUMBER: builtins.int
    COLS_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    rows: builtins.int
    cols: builtins.int
    @property
    def data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        rows: builtins.int = ...,
        cols: builtins.int = ...,
        data: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["cols", b"cols", "data", b"data", "rows", b"rows"]) -> None: ...

global___MatXF64 = MatXF64

@typing.final
class CameraSettings(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AUTO_EXPOSURE_FIELD_NUMBER: builtins.int
    EXPOSURE_TIME_FIELD_NUMBER: builtins.int
    ISO_VALUE_FIELD_NUMBER: builtins.int
    LENS_POS_FIELD_NUMBER: builtins.int
    auto_exposure: builtins.bool
    exposure_time: builtins.int
    iso_value: builtins.int
    lens_pos: builtins.int
    def __init__(
        self,
        *,
        auto_exposure: builtins.bool = ...,
        exposure_time: builtins.int = ...,
        iso_value: builtins.int = ...,
        lens_pos: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["auto_exposure", b"auto_exposure", "exposure_time", b"exposure_time", "iso_value", b"iso_value", "lens_pos", b"lens_pos"]) -> None: ...

global___CameraSettings = CameraSettings

@typing.final
class EncoderOptions(google.protobuf.message.Message):
    """The options for the video encoder"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROFILE_FIELD_NUMBER: builtins.int
    RATE_CONTROL_MODE_FIELD_NUMBER: builtins.int
    CBR_PREFERRED_BITRATE_KBPS_FIELD_NUMBER: builtins.int
    VBR_OR_MJPEG_QUALITY_FIELD_NUMBER: builtins.int
    FRAMES_PER_KEYFRAME_FIELD_NUMBER: builtins.int
    profile: global___EncoderProfile.ValueType
    """The profile to use for encoding."""
    rate_control_mode: global___RateControlMode.ValueType
    """The rate control mode to use."""
    cbr_preferred_bitrate_kbps: builtins.int
    """The preferred bitrate for constant bitrate (CBR) mode, in kilobits per second (kbps)."""
    vbr_or_mjpeg_quality: builtins.int
    """The quality setting for variable bitrate (VBR) mode or MJPEG encoding.
    For VBR mode, this represents the desired quality level.
    For MJPEG encoding, this represents the JPEG quality level.
    """
    frames_per_keyframe: builtins.int
    """The number of frames between keyframes."""
    def __init__(
        self,
        *,
        profile: global___EncoderProfile.ValueType = ...,
        rate_control_mode: global___RateControlMode.ValueType = ...,
        cbr_preferred_bitrate_kbps: builtins.int = ...,
        vbr_or_mjpeg_quality: builtins.int = ...,
        frames_per_keyframe: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["cbr_preferred_bitrate_kbps", b"cbr_preferred_bitrate_kbps", "frames_per_keyframe", b"frames_per_keyframe", "profile", b"profile", "rate_control_mode", b"rate_control_mode", "vbr_or_mjpeg_quality", b"vbr_or_mjpeg_quality"]) -> None: ...

global___EncoderOptions = EncoderOptions

@typing.final
class Resolution(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WIDTH_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    width: builtins.int
    height: builtins.int
    def __init__(
        self,
        *,
        width: builtins.int = ...,
        height: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["height", b"height", "width", b"width"]) -> None: ...

global___Resolution = Resolution

@typing.final
class OakImageMeta(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CATEGORY_FIELD_NUMBER: builtins.int
    INSTANCE_NUM_FIELD_NUMBER: builtins.int
    SEQUENCE_NUM_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMESTAMP_DEVICE_FIELD_NUMBER: builtins.int
    TIMESTAMP_RECV_FIELD_NUMBER: builtins.int
    SETTINGS_FIELD_NUMBER: builtins.int
    ENCODER_OPTIONS_FIELD_NUMBER: builtins.int
    RESOLUTION_FIELD_NUMBER: builtins.int
    PIXEL_FORMAT_FIELD_NUMBER: builtins.int
    category: builtins.int
    """DepthAI catetory"""
    instance_num: builtins.int
    """DepthAI instance number"""
    sequence_num: builtins.int
    """for synchronization between left, right and stereo"""
    timestamp: builtins.float
    """seconds, capture time (begin of exposure) synchronized with host monotonic"""
    timestamp_device: builtins.float
    """seconds, capture time (begin of exposure) in the device monotonic clock"""
    timestamp_recv: builtins.float
    """seconds, host receive time in host monotonic clock"""
    pixel_format: global___PixelFormat.ValueType
    @property
    def settings(self) -> global___CameraSettings: ...
    @property
    def encoder_options(self) -> global___EncoderOptions: ...
    @property
    def resolution(self) -> global___Resolution: ...
    def __init__(
        self,
        *,
        category: builtins.int = ...,
        instance_num: builtins.int = ...,
        sequence_num: builtins.int = ...,
        timestamp: builtins.float = ...,
        timestamp_device: builtins.float = ...,
        timestamp_recv: builtins.float = ...,
        settings: global___CameraSettings | None = ...,
        encoder_options: global___EncoderOptions | None = ...,
        resolution: global___Resolution | None = ...,
        pixel_format: global___PixelFormat.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["encoder_options", b"encoder_options", "resolution", b"resolution", "settings", b"settings"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["category", b"category", "encoder_options", b"encoder_options", "instance_num", b"instance_num", "pixel_format", b"pixel_format", "resolution", b"resolution", "sequence_num", b"sequence_num", "settings", b"settings", "timestamp", b"timestamp", "timestamp_device", b"timestamp_device", "timestamp_recv", b"timestamp_recv"]) -> None: ...

global___OakImageMeta = OakImageMeta

@typing.final
class OakFrame(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    META_FIELD_NUMBER: builtins.int
    IMAGE_DATA_FIELD_NUMBER: builtins.int
    image_data: builtins.bytes
    """decode/encode with cv2.imencode/cv2.imdecode"""
    @property
    def meta(self) -> global___OakImageMeta: ...
    def __init__(
        self,
        *,
        meta: global___OakImageMeta | None = ...,
        image_data: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["meta", b"meta"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["image_data", b"image_data", "meta", b"meta"]) -> None: ...

global___OakFrame = OakFrame

@typing.final
class OakGyro(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GYRO_FIELD_NUMBER: builtins.int
    SEQUENCE_NUM_FIELD_NUMBER: builtins.int
    ACCURACY_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMESTAMP_DEVICE_FIELD_NUMBER: builtins.int
    TIMESTAMP_RECV_FIELD_NUMBER: builtins.int
    sequence_num: builtins.int
    accuracy: builtins.str
    timestamp: builtins.float
    """See OakImageMeta"""
    timestamp_device: builtins.float
    """See OakImageMeta"""
    timestamp_recv: builtins.float
    """See OakImageMeta"""
    @property
    def gyro(self) -> global___Vec3F32: ...
    def __init__(
        self,
        *,
        gyro: global___Vec3F32 | None = ...,
        sequence_num: builtins.int = ...,
        accuracy: builtins.str = ...,
        timestamp: builtins.float = ...,
        timestamp_device: builtins.float = ...,
        timestamp_recv: builtins.float = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["gyro", b"gyro"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["accuracy", b"accuracy", "gyro", b"gyro", "sequence_num", b"sequence_num", "timestamp", b"timestamp", "timestamp_device", b"timestamp_device", "timestamp_recv", b"timestamp_recv"]) -> None: ...

global___OakGyro = OakGyro

@typing.final
class OakAccelero(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCELERO_FIELD_NUMBER: builtins.int
    SEQUENCE_NUM_FIELD_NUMBER: builtins.int
    ACCURACY_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMESTAMP_DEVICE_FIELD_NUMBER: builtins.int
    TIMESTAMP_RECV_FIELD_NUMBER: builtins.int
    sequence_num: builtins.int
    accuracy: builtins.str
    timestamp: builtins.float
    """See OakImageMeta"""
    timestamp_device: builtins.float
    """See OakImageMeta"""
    timestamp_recv: builtins.float
    """See OakImageMeta"""
    @property
    def accelero(self) -> global___Vec3F32: ...
    def __init__(
        self,
        *,
        accelero: global___Vec3F32 | None = ...,
        sequence_num: builtins.int = ...,
        accuracy: builtins.str = ...,
        timestamp: builtins.float = ...,
        timestamp_device: builtins.float = ...,
        timestamp_recv: builtins.float = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["accelero", b"accelero"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["accelero", b"accelero", "accuracy", b"accuracy", "sequence_num", b"sequence_num", "timestamp", b"timestamp", "timestamp_device", b"timestamp_device", "timestamp_recv", b"timestamp_recv"]) -> None: ...

global___OakAccelero = OakAccelero

@typing.final
class OakImuPacket(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    GYRO_PACKET_FIELD_NUMBER: builtins.int
    ACCELERO_PACKET_FIELD_NUMBER: builtins.int
    @property
    def gyro_packet(self) -> global___OakGyro: ...
    @property
    def accelero_packet(self) -> global___OakAccelero: ...
    def __init__(
        self,
        *,
        gyro_packet: global___OakGyro | None = ...,
        accelero_packet: global___OakAccelero | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["accelero_packet", b"accelero_packet", "gyro_packet", b"gyro_packet"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["accelero_packet", b"accelero_packet", "gyro_packet", b"gyro_packet"]) -> None: ...

global___OakImuPacket = OakImuPacket

@typing.final
class OakImuPackets(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PACKETS_FIELD_NUMBER: builtins.int
    @property
    def packets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OakImuPacket]: ...
    def __init__(
        self,
        *,
        packets: collections.abc.Iterable[global___OakImuPacket] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["packets", b"packets"]) -> None: ...

global___OakImuPackets = OakImuPackets

@typing.final
class OakTrackedFeaturePacket(google.protobuf.message.Message):
    """Message for a single tracked feature"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    XY_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    AGE_FIELD_NUMBER: builtins.int
    HARRISSCORE_FIELD_NUMBER: builtins.int
    TRACKINGERROR_FIELD_NUMBER: builtins.int
    id: builtins.int
    age: builtins.int
    harrisScore: builtins.float
    trackingError: builtins.float
    @property
    def xy(self) -> global___Vec2F32: ...
    def __init__(
        self,
        *,
        xy: global___Vec2F32 | None = ...,
        id: builtins.int = ...,
        age: builtins.int = ...,
        harrisScore: builtins.float = ...,
        trackingError: builtins.float = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["xy", b"xy"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["age", b"age", "harrisScore", b"harrisScore", "id", b"id", "trackingError", b"trackingError", "xy", b"xy"]) -> None: ...

global___OakTrackedFeaturePacket = OakTrackedFeaturePacket

@typing.final
class OakTrackedFeaturePackets(google.protobuf.message.Message):
    """Message for sending tracked features from the device to the host
    Reference: https://docs.luxonis.com/projects/api/en/latest/components/nodes/feature_tracker/
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PACKETS_FIELD_NUMBER: builtins.int
    SEQUENCE_NUM_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    TIMESTAMP_DEVICE_FIELD_NUMBER: builtins.int
    TIMESTAMP_RECV_FIELD_NUMBER: builtins.int
    sequence_num: builtins.int
    timestamp: builtins.float
    """See OakImageMeta"""
    timestamp_device: builtins.float
    """See OakImageMeta"""
    timestamp_recv: builtins.float
    """See OakImageMeta"""
    @property
    def packets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OakTrackedFeaturePacket]: ...
    def __init__(
        self,
        *,
        packets: collections.abc.Iterable[global___OakTrackedFeaturePacket] | None = ...,
        sequence_num: builtins.int = ...,
        timestamp: builtins.float = ...,
        timestamp_device: builtins.float = ...,
        timestamp_recv: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["packets", b"packets", "sequence_num", b"sequence_num", "timestamp", b"timestamp", "timestamp_device", b"timestamp_device", "timestamp_recv", b"timestamp_recv"]) -> None: ...

global___OakTrackedFeaturePackets = OakTrackedFeaturePackets

@typing.final
class OakDeviceInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    MXID_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    name: builtins.str
    mxid: builtins.str
    ip: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        mxid: builtins.str = ...,
        ip: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["ip", b"ip", "mxid", b"mxid", "name", b"name"]) -> None: ...

global___OakDeviceInfo = OakDeviceInfo

@typing.final
class OakSyncFrame(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEFT_FIELD_NUMBER: builtins.int
    RIGHT_FIELD_NUMBER: builtins.int
    RGB_FIELD_NUMBER: builtins.int
    DISPARITY_FIELD_NUMBER: builtins.int
    NN_FIELD_NUMBER: builtins.int
    IMU_PACKETS_FIELD_NUMBER: builtins.int
    SEQUENCE_NUM_FIELD_NUMBER: builtins.int
    DEVICE_INFO_FIELD_NUMBER: builtins.int
    sequence_num: builtins.int
    @property
    def left(self) -> global___OakFrame: ...
    @property
    def right(self) -> global___OakFrame: ...
    @property
    def rgb(self) -> global___OakFrame: ...
    @property
    def disparity(self) -> global___OakFrame: ...
    @property
    def nn(self) -> global___OakNNData: ...
    @property
    def imu_packets(self) -> global___OakImuPackets: ...
    @property
    def device_info(self) -> global___OakDeviceInfo: ...
    def __init__(
        self,
        *,
        left: global___OakFrame | None = ...,
        right: global___OakFrame | None = ...,
        rgb: global___OakFrame | None = ...,
        disparity: global___OakFrame | None = ...,
        nn: global___OakNNData | None = ...,
        imu_packets: global___OakImuPackets | None = ...,
        sequence_num: builtins.int = ...,
        device_info: global___OakDeviceInfo | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["device_info", b"device_info", "disparity", b"disparity", "imu_packets", b"imu_packets", "left", b"left", "nn", b"nn", "rgb", b"rgb", "right", b"right"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["device_info", b"device_info", "disparity", b"disparity", "imu_packets", b"imu_packets", "left", b"left", "nn", b"nn", "rgb", b"rgb", "right", b"right", "sequence_num", b"sequence_num"]) -> None: ...

global___OakSyncFrame = OakSyncFrame

@typing.final
class OakNNData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    META_FIELD_NUMBER: builtins.int
    NUM_CHANNELS_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    num_channels: builtins.int
    height: builtins.int
    width: builtins.int
    data: builtins.bytes
    @property
    def meta(self) -> global___OakImageMeta: ...
    def __init__(
        self,
        *,
        meta: global___OakImageMeta | None = ...,
        num_channels: builtins.int = ...,
        height: builtins.int = ...,
        width: builtins.int = ...,
        data: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["meta", b"meta"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["data", b"data", "height", b"height", "meta", b"meta", "num_channels", b"num_channels", "width", b"width"]) -> None: ...

global___OakNNData = OakNNData

@typing.final
class Pair(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    key: builtins.str
    value: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        value: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

global___Pair = Pair

@typing.final
class Metadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PAIRS_FIELD_NUMBER: builtins.int
    @property
    def pairs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Pair]: ...
    def __init__(
        self,
        *,
        pairs: collections.abc.Iterable[global___Pair] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["pairs", b"pairs"]) -> None: ...

global___Metadata = Metadata

@typing.final
class OakDataSample(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FRAME_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    @property
    def frame(self) -> global___OakSyncFrame: ...
    @property
    def metadata(self) -> global___Metadata: ...
    def __init__(
        self,
        *,
        frame: global___OakSyncFrame | None = ...,
        metadata: global___Metadata | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["frame", b"frame", "metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["frame", b"frame", "metadata", b"metadata"]) -> None: ...

global___OakDataSample = OakDataSample

@typing.final
class RotationMatrix(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROTATION_MATRIX_FIELD_NUMBER: builtins.int
    @property
    def rotation_matrix(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        rotation_matrix: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["rotation_matrix", b"rotation_matrix"]) -> None: ...

global___RotationMatrix = RotationMatrix

@typing.final
class Extrinsics(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROTATION_MATRIX_FIELD_NUMBER: builtins.int
    SPEC_TRANSLATION_FIELD_NUMBER: builtins.int
    TO_CAMERA_SOCKET_FIELD_NUMBER: builtins.int
    TRANSLATION_FIELD_NUMBER: builtins.int
    to_camera_socket: builtins.int
    @property
    def rotation_matrix(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def spec_translation(self) -> global___Vec3F32: ...
    @property
    def translation(self) -> global___Vec3F32: ...
    def __init__(
        self,
        *,
        rotation_matrix: collections.abc.Iterable[builtins.float] | None = ...,
        spec_translation: global___Vec3F32 | None = ...,
        to_camera_socket: builtins.int = ...,
        translation: global___Vec3F32 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["spec_translation", b"spec_translation", "translation", b"translation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["rotation_matrix", b"rotation_matrix", "spec_translation", b"spec_translation", "to_camera_socket", b"to_camera_socket", "translation", b"translation"]) -> None: ...

global___Extrinsics = Extrinsics

@typing.final
class CameraData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CAMERA_NUMBER_FIELD_NUMBER: builtins.int
    CAMERA_TYPE_FIELD_NUMBER: builtins.int
    DISTORTION_COEFF_FIELD_NUMBER: builtins.int
    EXTRINSICS_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    INTRINSIC_MATRIX_FIELD_NUMBER: builtins.int
    LENS_POSITION_FIELD_NUMBER: builtins.int
    SPEC_HFOV_DEG_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    camera_number: builtins.int
    camera_type: builtins.int
    height: builtins.int
    lens_position: builtins.int
    spec_hfov_deg: builtins.float
    width: builtins.int
    @property
    def distortion_coeff(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def extrinsics(self) -> global___Extrinsics: ...
    @property
    def intrinsic_matrix(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        camera_number: builtins.int = ...,
        camera_type: builtins.int = ...,
        distortion_coeff: collections.abc.Iterable[builtins.float] | None = ...,
        extrinsics: global___Extrinsics | None = ...,
        height: builtins.int = ...,
        intrinsic_matrix: collections.abc.Iterable[builtins.float] | None = ...,
        lens_position: builtins.int = ...,
        spec_hfov_deg: builtins.float = ...,
        width: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["extrinsics", b"extrinsics"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["camera_number", b"camera_number", "camera_type", b"camera_type", "distortion_coeff", b"distortion_coeff", "extrinsics", b"extrinsics", "height", b"height", "intrinsic_matrix", b"intrinsic_matrix", "lens_position", b"lens_position", "spec_hfov_deg", b"spec_hfov_deg", "width", b"width"]) -> None: ...

global___CameraData = CameraData

@typing.final
class StereoRectificationData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LEFT_CAMERA_SOCKET_FIELD_NUMBER: builtins.int
    RECTIFIED_ROTATION_LEFT_FIELD_NUMBER: builtins.int
    RECTIFIED_ROTATION_RIGHT_FIELD_NUMBER: builtins.int
    RIGHT_CAMERA_SOCKET_FIELD_NUMBER: builtins.int
    left_camera_socket: builtins.int
    right_camera_socket: builtins.int
    @property
    def rectified_rotation_left(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    @property
    def rectified_rotation_right(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        left_camera_socket: builtins.int = ...,
        rectified_rotation_left: collections.abc.Iterable[builtins.float] | None = ...,
        rectified_rotation_right: collections.abc.Iterable[builtins.float] | None = ...,
        right_camera_socket: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["left_camera_socket", b"left_camera_socket", "rectified_rotation_left", b"rectified_rotation_left", "rectified_rotation_right", b"rectified_rotation_right", "right_camera_socket", b"right_camera_socket"]) -> None: ...

global___StereoRectificationData = StereoRectificationData

@typing.final
class OakCalibration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BATCH_NAME_FIELD_NUMBER: builtins.int
    BATCH_TIME_FIELD_NUMBER: builtins.int
    BOARD_CONF_FIELD_NUMBER: builtins.int
    BOARD_CUSTOM_FIELD_NUMBER: builtins.int
    BOARD_NAME_FIELD_NUMBER: builtins.int
    BOARD_OPTIONS_FIELD_NUMBER: builtins.int
    BOARD_REV_FIELD_NUMBER: builtins.int
    CAMERA_DATA_FIELD_NUMBER: builtins.int
    HARDWARE_CONF_FIELD_NUMBER: builtins.int
    IMU_EXTRINSICS_FIELD_NUMBER: builtins.int
    MISCELLANEOUS_DATA_FIELD_NUMBER: builtins.int
    PRODUCT_NAME_FIELD_NUMBER: builtins.int
    STEREO_RECTIFICATION_DATA_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    batch_name: builtins.str
    batch_time: builtins.int
    board_conf: builtins.str
    board_custom: builtins.str
    board_name: builtins.str
    board_options: builtins.int
    board_rev: builtins.str
    hardware_conf: builtins.str
    product_name: builtins.str
    version: builtins.int
    @property
    def camera_data(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CameraData]: ...
    @property
    def imu_extrinsics(self) -> global___Extrinsics: ...
    @property
    def miscellaneous_data(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def stereo_rectification_data(self) -> global___StereoRectificationData: ...
    def __init__(
        self,
        *,
        batch_name: builtins.str = ...,
        batch_time: builtins.int = ...,
        board_conf: builtins.str = ...,
        board_custom: builtins.str = ...,
        board_name: builtins.str = ...,
        board_options: builtins.int = ...,
        board_rev: builtins.str = ...,
        camera_data: collections.abc.Iterable[global___CameraData] | None = ...,
        hardware_conf: builtins.str = ...,
        imu_extrinsics: global___Extrinsics | None = ...,
        miscellaneous_data: collections.abc.Iterable[builtins.str] | None = ...,
        product_name: builtins.str = ...,
        stereo_rectification_data: global___StereoRectificationData | None = ...,
        version: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["imu_extrinsics", b"imu_extrinsics", "stereo_rectification_data", b"stereo_rectification_data"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["batch_name", b"batch_name", "batch_time", b"batch_time", "board_conf", b"board_conf", "board_custom", b"board_custom", "board_name", b"board_name", "board_options", b"board_options", "board_rev", b"board_rev", "camera_data", b"camera_data", "hardware_conf", b"hardware_conf", "imu_extrinsics", b"imu_extrinsics", "miscellaneous_data", b"miscellaneous_data", "product_name", b"product_name", "stereo_rectification_data", b"stereo_rectification_data", "version", b"version"]) -> None: ...

global___OakCalibration = OakCalibration



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class HyperSpecImage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IMAGE_DATA_FIELD_NUMBER: builtins.int
    WIDTH_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    FRAME_RATE_FIELD_NUMBER: builtins.int
    BLOCKID_FIELD_NUMBER: builtins.int
    BANDWIDTH_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    image_data: builtins.bytes
    """Raw image data as bytes"""
    width: builtins.int
    """Width of the image"""
    height: builtins.int
    """Height of the image"""
    frame_rate: builtins.float
    """Frame rate"""
    blockid: builtins.int
    """Block ID"""
    bandwidth: builtins.int
    """Bandwidth"""
    timestamp: builtins.int
    """timestamp of image"""
    def __init__(
        self,
        *,
        image_data: builtins.bytes = ...,
        width: builtins.int = ...,
        height: builtins.int = ...,
        frame_rate: builtins.float = ...,
        blockid: builtins.int = ...,
        bandwidth: builtins.int = ...,
        timestamp: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["bandwidth", b"bandwidth", "blockid", b"blockid", "frame_rate", b"frame_rate", "height", b"height", "image_data", b"image_data", "timestamp", b"timestamp", "width", b"width"]) -> None: ...

global___HyperSpecImage = HyperSpecImage



import abc
from pathlib import Path
from typing import Any

import numpy as np
from numpy.typing import NDArray


class BasePreprocessor[T](abc.ABC):
    def __init__(self, path: str | Path, in_ext: str = "bin", **kwargs: Any) -> None:
        self._in_ext = self.process_ext(in_ext)
        self._base_name = self.validate_file_path(path)
        self.images: list[T] = []
        self.system_timestamps: list[int] = []

    @staticmethod
    def bytes_to_numpy(image: bytes) -> NDArray[np.uint8]:
        return np.frombuffer(image, dtype=np.uint8)

    def validate_file_path(self, path: str | Path) -> str:
        fpath = Path(path)
        if not fpath.exists():
            raise FileNotFoundError(f"File doesn't exist: {fpath!s}")
        if not fpath.is_file():
            raise ValueError(f"Not a file: {fpath!s}")
        self._path = fpath
        name = self.path.name
        if not name.endswith("." + self._in_ext):
            raise ValueError(f"Expects input file with ext: {self._in_ext}. Input: {name}")
        return name[: -len("." + self._in_ext)]

    def process_ext(self, ext: str) -> str:
        return ext[1:] if ext.startswith(".") else ext

    def get_output_name(self, index: int | None, ext: str, details: str | None = None) -> str:
        base = f"{self._base_name}_preproc"
        if index is not None:
            base += f"-{index}"
        if details is not None:
            base += f"_{details}"
        return f"{self._base_name}_preproc.{ext}"

    @abc.abstractmethod
    def extract(self, **kwargs: Any) -> None: ...

    @property
    def path(self) -> Path:
        return self._path
